package org.epistasis.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.NumberFormat;

import javax.swing.JProgressBar;

import org.epistasis.Stopwatch;

public class ProgressPanel extends TitledPanel {
    /**
	 * 
	 */
    private static final long serialVersionUID = 1L;
    private final JProgressBar prgProgress = new JProgressBar();
    private double fractionComplete = 0;
    private Stopwatch stopWatch;
    private final static NumberFormat percentFormatter = NumberFormat
	    .getPercentInstance();
    StringBuilder sb = new StringBuilder();

    public ProgressPanel() {
	super(new GridBagLayout());
	jbInit();
    }

    public double getValue() {
	return fractionComplete;
    }

    private void jbInit() {
	setTitle("Progress Completed");
	this.add(prgProgress, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
		GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
			0, 0, 0, 0), 240, 253));
	this.add(prgProgress, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
		GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
			5, 5, 5, 5), 0, 0));
	prgProgress.setStringPainted(true);
    }

    public void setFractionComplete(final double fractionComplete) {
	if (prgProgress.isIndeterminate()
		|| (fractionComplete == this.fractionComplete)
		|| ((fractionComplete >= 0) && (fractionComplete > 1.0))) {
	    return;
	}
	this.fractionComplete = fractionComplete;
	prgProgress.setValue(Math.round((float) fractionComplete * 100.0f));
	sb.setLength(0);
	sb.append(ProgressPanel.percentFormatter.format(fractionComplete));
	if (stopWatch != null) {
	    sb.append(" elapsed: ");
	    final long elapsedTimeMillis = stopWatch.getElapsedTimeMillis();
	    sb.append(stopWatch
		    .getElapsedTime(elapsedTimeMillis, false /* useReducedPrecision */));
	    final boolean isComplete = (fractionComplete >= 1);
	    // do not make estimate before we have enough data to be reasonably
	    // accurate
	    if (!isComplete
		    && ((elapsedTimeMillis > 10000) || (fractionComplete > 0.2))) {
		// if 5 seconds have passed try to estimate remaining
		// time
		final long estimatedTotalTime = (long) (elapsedTimeMillis / fractionComplete);
		sb.append(" remaining: ~");
		sb.append(stopWatch.getElapsedTime(estimatedTotalTime
			- elapsedTimeMillis, true /* useReducedPrecision */));
	    }
	    if (isComplete) {
		stopWatch = null;
	    }
	}
	prgProgress.setString(sb.toString());
	// System.err.println(sb.toString());
    }

    public void setIndeterminate(final boolean newValue) {
	prgProgress.setIndeterminate(newValue);
    }

    public void setString(final String stringToDisplay) {
	prgProgress.setString(stringToDisplay);
    }

    public void startShowingElapsedTime() {
	stopWatch = new Stopwatch(false /* showMilliseconds */);

    }
}
