package org.epistasis.gui;

public class ProgressPanelUpdater implements Runnable {
    private static final long REFRESH_INTERVAL_MILLIS = 1000;
    private long expectedNumberOfCalls;
    private long callCounter = 0;
    private long lastRefreshTimeMillis = -1;
    private final Runnable swingUpdater;

    public ProgressPanelUpdater(final ProgressPanel prgProgress) {
	this(prgProgress, false /* showElapsedTime */);
    }

    public ProgressPanelUpdater(final ProgressPanel prgProgress,
	    final boolean showElapsedTime) {
	if (showElapsedTime) {
	    prgProgress.startShowingElapsedTime();
	}
	swingUpdater = new SwingInvoker(new Runnable() {
	    @Override
	    public void run() {

		final double fractionComplete = callCounter
			/ (double) expectedNumberOfCalls;
		prgProgress.setFractionComplete(fractionComplete);
	    }
	});
    }

    public ProgressPanelUpdater(final ProgressPanel prgProgress, final long max) {
	this(prgProgress, max, false /* showElapsedTime */);
    }

    public ProgressPanelUpdater(final ProgressPanel prgProgress,
	    final long max, final boolean showElapsedTime) {
	this(prgProgress, showElapsedTime);
	setMaximum(max);
    }

    public ProgressPanelUpdater(final ProgressPanel prgProgress,
	    final long max, final boolean showElapsedTime,
	    final int refreshInterval) {
	this(prgProgress, showElapsedTime);
	setMaximum(max);
    }

    private synchronized long incrementCounter() {
	return ++callCounter;
    }

    @Override
    public void run() {
	final long currentValue = incrementCounter();
	final long currentTimeMillis = System.currentTimeMillis();
	final long elapsedTimeSinceRefresh = currentTimeMillis
		- lastRefreshTimeMillis;
	if ((elapsedTimeSinceRefresh >= ProgressPanelUpdater.REFRESH_INTERVAL_MILLIS)
		|| (currentValue >= expectedNumberOfCalls)) {
	    lastRefreshTimeMillis = currentTimeMillis;
	    swingUpdater.run();
	}
    }

    public void setMaximum(final long max) {
	expectedNumberOfCalls = max;
    }
}
